@@ -1,6 +1,7 @@
 import binascii
+import time
+import uuid
 from datetime import datetime
-from time import time
 
 from hazelcast import six
 from hazelcast.core import HazelcastJsonValue
@@ -10,6 +11,8 @@ from hazelcast.serialization.base import HazelcastSerializationError
 from hazelcast.serialization.serialization_const import *
 from hazelcast.six.moves import range, cPickle
 
+from hazelcast.util import to_signed
+
 if not six.PY2:
     long = int
 
@@ -24,7 +27,8 @@ class NoneSerializer(BaseSerializer):
     def read(self, inp):
         return None
 
-    # "write(self, out, obj)" is never called so not implemented here
+    def write(self, out, obj):
+        pass
 
     def get_type_id(self):
         return CONSTANT_TYPE_NULL
@@ -78,10 +82,7 @@ class IntegerSerializer(BaseSerializer):
         return inp.read_int()
 
     def write(self, out, obj):
-        if obj.bit_length() < 32:
-            out.write_int(obj)
-        else:
-            raise ValueError("Serialization only supports 32 bit ints")
+        out.write_int(obj)
 
     def get_type_id(self):
         return CONSTANT_TYPE_INTEGER
@@ -92,10 +93,7 @@ class LongSerializer(BaseSerializer):
         return inp.read_long()
 
     def write(self, out, obj):
-        if obj.bit_length() < 64:
-            out.write_long(obj)
-        else:
-            raise ValueError("Serialization only supports 64 bit longs")
+        out.write_long(obj)
 
     def get_type_id(self):
         return CONSTANT_TYPE_LONG
@@ -105,8 +103,7 @@ class FloatSerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_float()
 
-    def write(self, out, obj):
-        out.write_float(obj)
+    # "write(self, out, obj)" is never called so not implemented here
 
     def get_type_id(self):
         return CONSTANT_TYPE_FLOAT
@@ -125,21 +122,38 @@ class DoubleSerializer(BaseSerializer):
 
 class StringSerializer(BaseSerializer):
     def read(self, inp):
-        return inp.read_utf()
+        return inp.read_string()
 
     def write(self, out, obj):
-        out.write_utf(obj)
+        out.write_string(obj)
 
     def get_type_id(self):
         return CONSTANT_TYPE_STRING
 
 
-class HazelcastJsonValueSerializer(BaseSerializer):
+class UuidSerializer(BaseSerializer):
     def read(self, inp):
-        return HazelcastJsonValue(inp.read_utf())
+        msb = inp.read_long()
+        lsb = inp.read_long()
+        return uuid.UUID(int=(((msb << UUID_MSB_SHIFT) & UUID_MSB_MASK) | (lsb & UUID_LSB_MASK)))
 
     def write(self, out, obj):
-        out.write_utf(obj.to_string())
+        i = obj.int
+        msb = to_signed(i >> UUID_MSB_SHIFT, 64)
+        lsb = to_signed(i & UUID_LSB_MASK, 64)
+        out.write_long(msb)
+        out.write_long(lsb)
+
+    def get_type_id(self):
+        return CONSTANT_TYPE_UUID
+
+
+class HazelcastJsonValueSerializer(BaseSerializer):
+    def read(self, inp):
+        return HazelcastJsonValue(inp.read_string())
+
+    def write(self, out, obj):
+        out.write_string(obj.to_string())
 
     def get_type_id(self):
         return JAVASCRIPT_JSON_SERIALIZATION_TYPE
@@ -160,7 +174,8 @@ class ByteArraySerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_byte_array()
 
-    # "write(self, out, obj)" is never called so not implemented here
+    def write(self, out, obj):
+        out.write_byte_array(obj)
 
     def get_type_id(self):
         return CONSTANT_TYPE_BYTE_ARRAY
@@ -228,7 +243,7 @@ class DoubleArraySerializer(BaseSerializer):
 
 class StringArraySerializer(BaseSerializer):
     def read(self, inp):
-        return inp.read_utf_array()
+        return inp.read_string_array()
 
     # "write(self, out, obj)" is never called so not implemented here
 
@@ -243,7 +258,7 @@ class DateTimeSerializer(BaseSerializer):
         return datetime.fromtimestamp(long_time / 1000.0)
 
     def write(self, out, obj):
-        long_time = long(time.mktime(obj.timetuple()))
+        long_time = long(time.mktime(obj.timetuple())) * 1000
         out.write_long(long_time)
 
     def get_type_id(self):
@@ -266,11 +281,11 @@ class BigIntegerSerializer(BaseSerializer):
         return int(binascii.hexlify(result), 16)
 
     def write(self, out, obj):
-        the_big_int = -obj-1 if obj < 0 else obj
+        the_big_int = -obj - 1 if obj < 0 else obj
         end_index = -1 if (type(obj) == long and six.PY2) else None
         hex_str = hex(the_big_int)[2:end_index]
         if len(hex_str) % 2 == 1:
-            prefix = '0' # "f" if obj < 0 else "0"
+            prefix = "0"  # "f" if obj < 0 else "0"
             hex_str = prefix + hex_str
         num_array = bytearray(binascii.unhexlify(bytearray(hex_str, encoding="utf-8")))
         if obj < 0:
@@ -284,45 +299,16 @@ class BigIntegerSerializer(BaseSerializer):
         return JAVA_DEFAULT_TYPE_BIG_INTEGER
 
 
-class BigDecimalSerializer(BaseSerializer):
-    def read(self, inp):
-        raise NotImplementedError("Big decimal numbers not supported")
-
-    def write(self, out, obj):
-        raise NotImplementedError("Big decimal numbers not supported")
-
-    def get_type_id(self):
-        return JAVA_DEFAULT_TYPE_BIG_DECIMAL
-
-
 class JavaClassSerializer(BaseSerializer):
     def read(self, inp):
-        return inp.read_utf()
+        return inp.read_string()
 
-    def write(self, out, obj):
-        out.write_utf(obj)
+    # "write(self, out, obj)" is never called so not implemented here
 
     def get_type_id(self):
         return JAVA_DEFAULT_TYPE_CLASS
 
 
-class JavaEnumSerializer(BaseSerializer):
-    def read(self, inp):
-        """
-        :param inp:
-        :return: a tuple of (Enum-name, Enum-value-name)
-        """
-        return tuple(inp.read_utf(), inp.read_utf())
-
-    def write(self, out, obj):
-        enum_name, enum_val_name = obj
-        out.write_utf(enum_name)
-        out.write_utf(enum_val_name)
-
-    def get_type_id(self):
-        return JAVA_DEFAULT_TYPE_ENUM
-
-
 class ArrayListSerializer(BaseSerializer):
     def read(self, inp):
         size = inp.read_int()
@@ -347,8 +333,7 @@ class LinkedListSerializer(BaseSerializer):
             return [inp.read_object() for _ in range(0, size)]
         return None
 
-    def write(self, out, obj):
-        raise NotImplementedError("writing Link lists not supported")
+    # "write(self, out, obj)" is never called so not implemented here
 
     def get_type_id(self):
         return JAVA_DEFAULT_TYPE_LINKED_LIST
@@ -356,11 +341,11 @@ class LinkedListSerializer(BaseSerializer):
 
 class PythonObjectSerializer(BaseSerializer):
     def read(self, inp):
-        str = inp.read_utf().encode()
+        str = inp.read_string().encode()
         return cPickle.loads(str)
 
     def write(self, out, obj):
-        out.write_utf(cPickle.dumps(obj, 0).decode("utf-8"))
+        out.write_string(cPickle.dumps(obj, 0).decode("utf-8"))
 
     def get_type_id(self):
         return PYTHON_TYPE_PICKLE
@@ -379,17 +364,23 @@ class IdentifiedDataSerializer(BaseSerializer):
     def read(self, inp):
         is_identified = inp.read_boolean()
         if not is_identified:
-            raise HazelcastSerializationError("Native clients only support IdentifiedDataSerializable!")
+            raise HazelcastSerializationError(
+                "Native clients only support IdentifiedDataSerializable!"
+            )
         factory_id = inp.read_int()
         class_id = inp.read_int()
 
         factory = self._factories.get(factory_id, None)
         if factory is None:
-            raise HazelcastSerializationError("No DataSerializerFactory registered for namespace: {}".format(factory_id))
+            raise HazelcastSerializationError(
+                "No DataSerializerFactory registered for namespace: %s" % factory_id
+            )
         identified = factory.get(class_id, None)
         if identified is None:
             raise HazelcastSerializationError(
-                    "{} is not be able to create an instance for id: {} on factoryId: {}".format(factory, class_id, factory_id))
+                "%s is not be able to create an instance for id: %s on factoryId: %s"
+                % (factory, class_id, factory_id)
+            )
         instance = identified()
         instance.read_data(inp)
         return instance
